@@ -183,7 +183,7 @@ rb_audiocd_source_new (RBPlugin *plugin,
 			       "volume", volume,
 			       "shell", shell,
 			       "sorting-key", NULL,
-			       "sourcelist-group", RB_SOURCELIST_GROUP_REMOVABLE,
+			       "source-group", RB_SOURCE_GROUP_DEVICES,
 			       "plugin", plugin,
 			       NULL);
 
@@ -266,21 +266,10 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source,
 		GstFormat track_format = gst_format_get_by_nick ("track");
 		gint64 duration;
 		gboolean result;
-#ifdef HAVE_GSTREAMER_0_8
-		GstEvent *event;
-
-		event = gst_event_new_seek (track_format | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH,
-					    (guint64) track_number - 1);
-		result = gst_element_send_event (priv->fakesink, event);
-
-		if (result)
-			result = gst_element_query (priv->fakesink, GST_QUERY_TOTAL, &time_format, &duration);
-#elif HAVE_GSTREAMER_0_10
                 result = gst_element_seek (priv->fakesink, 1.0, track_format, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (guint64) track_number - 1, GST_SEEK_TYPE_NONE, -1);
 		if (result) {
 			result = gst_element_query_duration (priv->fakesink, &time_format, &duration) && time_format == GST_FORMAT_TIME;
 		}
-#endif
 
 		if (result) {
 			g_value_init (&value, G_TYPE_ULONG);
@@ -311,17 +300,10 @@ rb_audiocd_get_cd_info (RBAudioCdSource *source,
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	GstFormat fmt = gst_format_get_by_nick ("track");
-#ifdef HAVE_GSTREAMER_0_8
-	if (!gst_element_query (priv->cdda, GST_QUERY_TOTAL, &fmt, num_tracks)) {
-		rb_debug ("failed to read cd track count");
-		return FALSE;
-	}
-#elif HAVE_GSTREAMER_0_10
 	GstFormat out_fmt = fmt;
 	if (!gst_element_query_duration (priv->cdda, &out_fmt, num_tracks) || out_fmt != fmt) {
 		return FALSE;
 	}
-#endif
 
 	return TRUE;
 }
@@ -332,18 +314,9 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 {
 	gint64 i, num_tracks;
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
-#ifdef HAVE_GSTREAMER_0_10
         GstStateChangeReturn ret;
-#endif
 	gboolean ok = TRUE;
 
-#ifdef HAVE_GSTREAMER_0_8
-	if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED) != GST_STATE_SUCCESS) {
-		rb_error_dialog (NULL, _("Couldn't load Audio CD"),
-					_("Rhythmbox couldn't access the CD."));
-		ok = FALSE;
-	}
-#elif HAVE_GSTREAMER_0_10
 	ret = gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
 	if (ret == GST_STATE_CHANGE_ASYNC) {
 		ret = gst_element_get_state (priv->pipeline, NULL, NULL, 3 * GST_SECOND);
@@ -353,7 +326,6 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 					_("Rhythmbox couldn't access the CD."));
 		ok = FALSE;
 	}
-#endif
 
 	if (ok && !rb_audiocd_get_cd_info (source, &num_tracks)) {
 		rb_error_dialog (NULL, _("Couldn't load Audio CD"),
@@ -374,11 +346,7 @@ rb_audiocd_scan_songs (RBAudioCdSource *source,
 		priv->tracks = g_list_reverse (priv->tracks);
 	}
 
-#ifdef HAVE_GSTREAMER_0_8
-	if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) != GST_STATE_SUCCESS) {
-#elif HAVE_GSTREAMER_0_10
 	if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-#endif
 		rb_debug ("failed to set cd state");
 	}
 
@@ -568,12 +536,23 @@ metadata_cb (SjMetadata *metadata,
 
 		rb_debug ("storing metadata for %s - %s - %s", track->artist, album->title, track->title);
 
+		rb_debug ("genre: %s", album->genre);
+		rb_debug ("musicbrainz_trackid: %s", track->track_id);
+		rb_debug ("musicbrainz_artistid: %s", track->artist_id);
+		rb_debug ("musicbrainz_albumid: %s", album->album_id);
+		rb_debug ("musicbrainz_albumartistid: %s", album->artist_id);
+		rb_debug ("musicbrainz_sortname: %s", album->artist_sortname);
+
 		/* record track info in entry*/
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_TITLE, track->title);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ARTIST, track->artist);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_ALBUM, album->title);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_GENRE, album->genre);
 		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID, track->track_id);
+		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID, track->artist_id);
+		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID, album->album_id);
+		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID, album->artist_id);
+		entry_set_string_prop (db, entry, TRUE, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME, album->artist_sortname);
 
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, track->duration);
@@ -679,12 +658,7 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 	}
 
 	rb_debug ("cdda longname: %s", gst_element_factory_get_longname (gst_element_get_factory (priv->cdda)));
-#ifdef HAVE_GSTREAMER_0_8
-	gst_element_set (priv->cdda, "device", priv->device_path, NULL);
-#elif HAVE_GSTREAMER_0_10
 	g_object_set (G_OBJECT (priv->cdda), "device", priv->device_path, NULL);
-#endif
-
 	priv->pipeline = gst_pipeline_new ("pipeline");
 	priv->fakesink = gst_element_factory_make ("fakesink", "fakesink");
 	gst_bin_add_many (GST_BIN (priv->pipeline), priv->cdda, priv->fakesink, NULL);
