@@ -73,6 +73,8 @@
 #define RB_LASTFM_VERSION "1.1.1"
 #define EXTRA_URI_ENCODE_CHARS	"&+"
 
+#define LAST_FM_NO_COVER_IMAGE "http://static.last.fm/depth/catalogue/noimage/cover_med.gif"
+
 
 static void rb_lastfm_source_class_init (RBLastfmSourceClass *klass);
 static void rb_lastfm_source_init (RBLastfmSource *source);
@@ -130,18 +132,18 @@ static char *rb_lastfm_source_title_from_uri (char *uri);
 static void rb_lastfm_source_add_station_cb (GtkButton *button, gpointer *data);
 static void rb_lastfm_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry, RBLastfmSource *source);
 
-static void rb_lastfm_source_new_song_cb (GObject *player_backend, gpointer data, RBLastfmSource *source);
+static void rb_lastfm_source_new_song_cb (GObject *player_backend,
+					  gpointer stream_data,
+					  gpointer data,
+					  RBLastfmSource *source);
 static void rb_lastfm_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBLastfmSource *source);
 
 static void show_entry_popup (RBEntryView *view,
 			      gboolean over_entry,
 			      RBSource *source);
 
-#ifdef HAVE_GSTREAMER_0_10
 /* can't be bothered creating a whole header file just for this: */
 GType rb_lastfm_src_get_type (void);
-#endif
-
 
 static const char* const radio_options[][3] = {
 	{N_("Similar Artists radio"), "lastfm://artist/%s/similarartists", N_("Artists similar to %s")},
@@ -263,9 +265,7 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_type_class_add_private (klass, sizeof (RBLastfmSourcePrivate));
 
-#ifdef HAVE_GSTREAMER_0_10
 	rb_lastfm_src_get_type ();
-#endif
 }
 
 static void
@@ -518,6 +518,7 @@ rb_lastfm_source_new (RBShell *shell)
 					  "shell", shell,
 					  "entry-type", entry_type,
 					  "proxy-config", proxy_config,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 
@@ -1057,6 +1058,10 @@ rb_lastfm_source_title_from_uri (char *uri)
 		title = g_strdup_printf (_("%s's %s Radio"), data[3], data[4]);
 	}
 
+	if (title == NULL && strcmp(data[2], "group") == 0) {
+		title = g_strdup_printf (_("%s Group Radio"), data[3]);
+	}
+
 	if (title == NULL) {
 		title = g_strdup (g_strstrip (uri));
 	}
@@ -1135,6 +1140,7 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 	char **pieces;
 	int p;
 	RhythmDBEntry *entry;
+	gboolean found_cover;
 
 	entry = rb_shell_player_get_playing_entry (source->priv->shell_player);
 	if (entry == NULL || rhythmdb_entry_get_entry_type (entry) != source->priv->entry_type) {
@@ -1148,6 +1154,7 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 
 	g_strstrip (body);
 	pieces = g_strsplit (body, "\n", 0);
+	found_cover = FALSE;
 
 	for (p = 0; pieces[p] != NULL; p++) {
 		gchar **values;
@@ -1168,6 +1175,22 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 			rb_streaming_source_set_streaming_title (RB_STREAMING_SOURCE (source), values[1]);
 		} else if (strcmp (values[0], "albumcover_small") == 0) {
 		} else if (strcmp (values[0], "albumcover_medium") == 0) {
+			GValue v = {0,};
+
+			rb_debug ("albumcover -> %s", values[1]);
+			if (strcmp (values[1], LAST_FM_NO_COVER_IMAGE) == 0) {
+				rb_debug ("ignoring last.fm's no cover image");
+			} else {
+				g_value_init (&v, G_TYPE_STRING);
+				g_value_set_string (&v, values[1]);
+				rhythmdb_emit_entry_extra_metadata_notify (source->priv->db,
+									   entry,
+									   "rb:coverArt-uri",
+									   &v);
+				g_value_unset (&v);
+				found_cover = TRUE;
+			}
+
 		} else if (strcmp (values[0], "albumcover_large") == 0) {
 		} else if (strcmp (values[0], "trackprogress") == 0) {
 		} else if (strcmp (values[0], "trackduration") == 0) {
@@ -1185,12 +1208,26 @@ rb_lastfm_source_metadata_cb (SoupMessage *req, RBLastfmSource *source)
 	g_strfreev (pieces);
 	g_free (body);
 
+	if (found_cover == FALSE) {
+		GValue v = {0,};
+
+		/* art display plugin treats an empty URI as meaning 'no cover art' */
+		g_value_init (&v, G_TYPE_STRING);
+		g_value_set_string (&v, "");
+		rhythmdb_emit_entry_extra_metadata_notify (source->priv->db,
+							   entry,
+							   "rb:coverArt-uri",
+							   &v);
+		g_value_unset (&v);
+	}
+
 	source->priv->status = OK;
 	rb_source_notify_status_changed (RB_SOURCE (source));
 }
 
 static void
 rb_lastfm_source_new_song_cb (GObject *player_backend,
+			      gpointer stream_data,
 			      gpointer data,
 			      RBLastfmSource *source)
 {
