@@ -45,11 +45,6 @@
 #include "rhythmdb.h"
 #include "rb-cut-and-paste-code.h"
 
-#ifdef IPOD_SUPPORT
-#define PHONE_VENDOR_ID 0x22b8
-#define PHONE_PRODUCT_ID 0x4810
-#endif
-
 static GObject *rb_ipod_source_constructor (GType type,
 					    guint n_construct_properties,
 					    GObjectConstructParam *construct_properties);
@@ -87,6 +82,7 @@ static gchar* ipod_path_from_unix_path (const gchar *mount_point,
 					const gchar *unix_path);
 #endif
 static void itdb_schedule_save (RBiPodSource *source);
+static RhythmDB *get_db_for_source (RBiPodSource *source);
 
 typedef struct
 {
@@ -94,10 +90,13 @@ typedef struct
 	gchar *ipod_mount_path;
 	GHashTable *entry_map;
 
-	GList *playlists;
 	gboolean needs_shuffle_db;
+	RBStaticPlaylistSource *podcast_pl;
 
 	guint load_idle_id;
+
+	GHashTable *artwork_request_map;
+	guint artwork_notify_id;
 } RBiPodSourcePrivate;
 
 RB_PLUGIN_DEFINE_TYPE(RBiPodSource,
@@ -118,8 +117,8 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
-   	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
-   	source_class->impl_move_to_trash = impl_move_to_trash;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_move_to_trash = impl_move_to_trash;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
@@ -206,7 +205,7 @@ rb_ipod_source_dispose (GObject *object)
  	if (priv->ipod_db != NULL) {
  		itdb_free (priv->ipod_db);
  		priv->ipod_db = NULL;
-  	}
+	}
 
 	if (priv->ipod_mount_path) {
 		g_free (priv->ipod_mount_path);
@@ -223,6 +222,18 @@ rb_ipod_source_dispose (GObject *object)
 		priv->load_idle_id = 0;
 	}
 
+	if (priv->artwork_request_map) {
+		g_hash_table_destroy (priv->artwork_request_map);
+		priv->artwork_request_map = NULL;
+ 	}
+
+	if (priv->artwork_notify_id) {
+		RhythmDB *db = get_db_for_source (RB_IPOD_SOURCE (object));
+		g_signal_handler_disconnect (db, priv->artwork_notify_id);
+		priv->artwork_notify_id = 0;
+		g_object_unref (db);
+	}
+
 	G_OBJECT_CLASS (rb_ipod_source_parent_class)->dispose (object);
 }
 
@@ -249,11 +260,11 @@ rb_ipod_source_new (RBShell *shell,
 	g_free (path);
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
-					  "entry-type", entry_type,
-					  "volume", volume,
-					  "shell", shell,
-					  "sourcelist-group", RB_SOURCELIST_GROUP_REMOVABLE,
-					  NULL));
+					       "entry-type", entry_type,
+					       "volume", volume,
+					       "shell", shell,
+					       "source-group", RB_SOURCE_GROUP_DEVICES,
+					       NULL));
 
 	rb_shell_register_entry_type_for_source (shell, RB_SOURCE (source), entry_type);
 
@@ -291,6 +302,42 @@ ipod_path_to_uri (const char *mount_point, const char *ipod_path)
  	return uri;
 }
 
+static void
+playlist_track_removed (RhythmDBQueryModel *m,
+			RhythmDBEntry *entry,
+			gpointer data)
+{
+	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
+	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	Itdb_Track *track;
+        track = g_hash_table_lookup (priv->entry_map, entry);
+	g_return_if_fail (track != NULL);
+
+	itdb_playlist_remove_track (ipod_pl, track);
+	itdb_schedule_save (ipod);
+}
+
+static void
+playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
+		      GtkTreeIter *iter, gpointer data)
+{
+	RBStaticPlaylistSource *playlist = RB_STATIC_PLAYLIST_SOURCE (data);
+	Itdb_Playlist *ipod_pl = g_object_get_data (G_OBJECT (playlist), "itdb-playlist");
+	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
+	Itdb_Track *track;
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+        track = g_hash_table_lookup (priv->entry_map, entry);
+	g_return_if_fail (track != NULL);
+
+	itdb_playlist_add_track (ipod_pl, track, -1);
+	itdb_schedule_save (ipod);
+}
+
 static void
 add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 {
@@ -299,11 +346,12 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
+		RhythmDBQueryModel *model;
 
-  	g_object_get (source,
-		      "shell", &shell,
-		      "entry-type", &entry_type,
-		      NULL);
+	g_object_get (source,
+			  "shell", &shell,
+			  "entry-type", &entry_type,
+			  NULL);
 
 	playlist_source = rb_static_playlist_source_new (shell,
 							 playlist->name,
@@ -323,8 +371,25 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 		g_free (filename);
 	}
 
-	priv->playlists = g_list_prepend (priv->playlists, playlist_source);
+	g_object_ref (G_OBJECT(playlist_source));
+		playlist->userdata = playlist_source;
+		playlist->userdata_destroy = g_object_unref;
+		playlist->userdata_duplicate = g_object_ref;
 
+		model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
+		g_signal_connect (model, "row-inserted",
+				          G_CALLBACK (playlist_track_added),
+				          playlist_source);
+		g_signal_connect (model, "entry-removed",
+				          G_CALLBACK (playlist_track_removed),
+				          playlist_source);
+
+		g_object_set_data (G_OBJECT (playlist_source),
+				           "ipod-source", source);
+		g_object_set_data (G_OBJECT (playlist_source),
+				           "itdb-playlist", playlist);
+		if (itdb_playlist_is_podcasts(playlist))
+				priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
 	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
 	g_object_unref (shell);
 }
@@ -352,6 +417,11 @@ load_ipod_playlists (RBiPodSource *source)
 }
 
 #ifdef ENABLE_IPOD_WRITING
+
+/* FIXME:  these should go away once we compile against new-enough libgpod */
+#define MEDIATYPE_AUDIO         0x0001
+#define MEDIATYPE_PODCAST       0x0004
+
 static Itdb_Track *
 create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 {
@@ -378,6 +448,12 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->app_rating = track->rating;
 	track->playcount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 
+	if (rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+		track->mediatype = MEDIATYPE_PODCAST;
+	} else {
+		track->mediatype = MEDIATYPE_AUDIO;
+	}
+
 	return track;
 }
 #endif
@@ -544,9 +620,9 @@ get_db_for_source (RBiPodSource *source)
 	RBShell *shell;
 	RhythmDB *db;
 
-  	g_object_get (source, "shell", &shell, NULL);
-  	g_object_get (shell, "db", &db, NULL);
-  	g_object_unref (shell);
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
+	g_object_unref (shell);
 
 	return db;
 }
@@ -562,7 +638,7 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 
 	db = get_db_for_source (source);
 
-  	g_assert (db != NULL);
+	g_assert (db != NULL);
  	for (it = priv->ipod_db->tracks; it != NULL; it = it->next) {
 		add_ipod_song_to_db (source, db, (Itdb_Track *)it->data);
 	}
@@ -587,7 +663,6 @@ rb_ipod_load_songs (RBiPodSource *source)
 	priv->ipod_mount_path = rb_ipod_get_mount_path (volume);
 
  	priv->ipod_db = itdb_parse (priv->ipod_mount_path, NULL);
-	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	info = itdb_device_get_ipod_info(priv->ipod_db->device);
 	if (info->ipod_generation == ITDB_IPOD_GENERATION_UNKNOWN ||
@@ -597,6 +672,7 @@ rb_ipod_load_songs (RBiPodSource *source)
 		priv->needs_shuffle_db = FALSE;
 	}
 
+	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
 		Itdb_Playlist *mpl;
 
@@ -646,13 +722,7 @@ rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
 		return NULL;
 	}
 
-#ifdef IPOD_SUPPORT
-	result = itdb_get_itunesdb_path (mount_point);
-#else
-	result = g_build_filename (mount_point,
-				   "iPod_Control/iTunes/iTunesDB",
-				   NULL);
-#endif
+	result = itdb_get_itunes_dir(mount_point);
 
 	g_free (mount_point);
 	return result;
@@ -702,7 +772,7 @@ rb_ipod_is_volume_ipod (GnomeVFSVolume *volume)
 	return rb_ipod_volume_has_ipod_db (volume);
 }
 
-#ifdef HAVE_HAL_0_5
+#ifdef HAVE_HAL
 
 static gboolean
 hal_udi_is_ipod (const char *udi)
@@ -746,48 +816,6 @@ hal_udi_is_ipod (const char *udi)
 
 	parent_name = libhal_device_get_property_string (ctx, parent_udi,
 			"storage.model", &error);
-#ifdef IPOD_SUPPORT
-	{
-		char *spider_udi;
-		int vnd_id = 0;
-		int product_id = 0;
-
-		spider_udi = g_strdup(parent_udi);
-		while (vnd_id == 0 && product_id == 0 && spider_udi != NULL) {
-			char *old_udi = spider_udi;
-			spider_udi =  libhal_device_get_property_string (ctx, spider_udi,
-					"info.parent", &error);
-			if (dbus_error_is_set (&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				spider_udi = NULL;
-				break;
-			}
-			g_free(old_udi);
-
-			vnd_id = libhal_device_get_property_int (ctx, spider_udi,
-				"usb.vendor_id", &error);
-			if (dbus_error_is_set(&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				vnd_id = 0;
-			}
-
-			product_id = libhal_device_get_property_int (ctx, spider_udi,
-				"usb.product_id", &error);
-			if (dbus_error_is_set(&error)) {
-				dbus_error_free (&error);
-				dbus_error_init (&error);
-				product_id = 0;
-			}
-		}
-		g_free (spider_udi);
-
-		if (vnd_id == PHONE_VENDOR_ID && product_id == PHONE_PRODUCT_ID) {
-			result = TRUE;
-		}
-	}
-#endif
 	if (parent_name == NULL || dbus_error_is_set (&error))
 		goto end;
 
@@ -814,41 +842,6 @@ end:
 
 	return result;
 }
-
-#elif HAVE_HAL_0_2
-
-static gboolean
-hal_udi_is_ipod (const char *udi)
-{
-	LibHalContext *ctx;
-	char *parent_udi;
-	char *parent_name;
-	gboolean result;
-
-	result = FALSE;
-	ctx = hal_initialize (NULL, FALSE);
-	if (ctx == NULL) {
-		/* FIXME: should we return an error somehow so that we can
-		 * fall back to a check for iTunesDB presence instead ?
-		 */
-		return FALSE;
-	}
-	parent_udi = hal_device_get_property_string (ctx, udi,
-			"info.parent");
-	parent_name = hal_device_get_property_string (ctx, parent_udi,
-			"storage.model");
-	g_free (parent_udi);
-
-	if (parent_name != NULL && strcmp (parent_name, "iPod") == 0) {
-		result = TRUE;
-	}
-
-	g_free (parent_name);
-	hal_shutdown (ctx);
-
-	return result;
-}
-
 #endif
 
 static GList*
@@ -913,10 +906,10 @@ impl_move_to_trash (RBSource *asource)
 	}
 
 	if (sel != NULL) {
-                itdb_schedule_save (RB_IPOD_SOURCE (asource));
+		itdb_schedule_save (RB_IPOD_SOURCE (asource));
 	}
 
-  	g_object_unref (db);
+	g_object_unref (db);
 
 	g_list_free (sel);
 }
@@ -936,7 +929,7 @@ itdb_schedule_save (RBiPodSource *source)
 	itdb_write (priv->ipod_db, NULL);
 
 	if (priv->needs_shuffle_db) {
-       		itdb_shuffle_write (priv->ipod_db, NULL);
+		itdb_shuffle_write (priv->ipod_db, NULL);
 	}
 }
 
@@ -964,6 +957,96 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	return NULL;
 }
 
+static void
+artwork_notify_cb (RhythmDB *db,
+		   RhythmDBEntry *entry,
+		   const gchar *property_name,
+		   const GValue *metadata,
+		   RBiPodSource *isource)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
+	Itdb_Track *song;
+#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
+#else /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
+	gchar *image_data;
+	gsize image_data_len;
+	GError *err = NULL;
+	gboolean success;
+#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
+	
+	song = g_hash_table_lookup (priv->artwork_request_map, entry);
+	if (song == NULL)
+		return;
+
+	g_return_if_fail (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF));
+#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
+	itdb_track_set_thumbnails_from_pixbuf (song, g_value_get_object (metadata));
+#else /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
+	success = gdk_pixbuf_save_to_buffer (GDK_PIXBUF (g_value_get_object (metadata)),
+					     &image_data, &image_data_len,
+					     "jpeg", &err,
+					     "quality", "100",
+					     NULL);
+	if (!success) {
+		g_assert (image_data == NULL);
+		g_warning ("Failed to save pixbuf to buffer %s", err->message);
+		g_error_free (err);
+		return;
+	}
+
+	g_hash_table_remove (priv->artwork_request_map, entry);
+	itdb_track_set_thumbnails_from_data (song, (guchar *) image_data, image_data_len);
+	g_free (image_data);
+#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
+	itdb_schedule_save (isource);
+}
+
+static void
+request_artwork (RBiPodSource *isource,
+		 RhythmDBEntry *entry,
+		 RhythmDB *db,
+		 Itdb_Track *song)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
+	GValue *metadata;
+
+	if (priv->artwork_request_map == NULL) {
+		priv->artwork_request_map = g_hash_table_new (g_direct_hash, g_direct_equal);
+	}
+
+	g_hash_table_insert (priv->artwork_request_map, entry, song);
+
+	if (priv->artwork_notify_id == 0) {
+		priv->artwork_notify_id = g_signal_connect_object (db, "entry-extra-metadata-notify::rb:coverArt",
+								   (GCallback)artwork_notify_cb, isource, 0);
+	}
+
+	metadata = rhythmdb_entry_request_extra_metadata (db, entry, "rb:coverArt");
+	if (metadata) {
+		artwork_notify_cb (db, entry, "rb:coverArt", metadata, isource);
+	}
+}
+
+static void
+add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
+{
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+	gchar *filename;
+
+	if (priv->podcast_pl == NULL) {
+		/* No Podcast playlist on the iPod, create a new one */
+		Itdb_Playlist *ipod_playlist;
+		ipod_playlist = itdb_playlist_new (_("Podcasts"), FALSE);
+		itdb_playlist_set_podcasts (ipod_playlist);
+		itdb_playlist_add (priv->ipod_db, ipod_playlist, -1);
+		add_rb_playlist (source, ipod_playlist);
+	}
+
+  	filename = ipod_path_to_uri (priv->ipod_mount_path, song->ipod_path);
+ 	rb_static_playlist_source_add_location (priv->podcast_pl, filename, -1);
+	g_free (filename);
+}
+
 static gboolean
 impl_track_added (RBRemovableMediaSource *source,
 		  RhythmDBEntry *entry,
@@ -987,7 +1070,19 @@ impl_track_added (RBRemovableMediaSource *source,
 		itdb_track_add (priv->ipod_db, song, -1);
 		itdb_playlist_add_track (itdb_playlist_mpl (priv->ipod_db),
 					 song, -1);
-
+		if (song->mediatype == MEDIATYPE_PODCAST) {
+			add_to_podcasts (isource, song);
+		}
+#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
+		/* reuse that #define since both functions were added to 
+		 * libgpod CVS HEAD around the same time
+		 */
+		if (itdb_device_supports_artwork (priv->ipod_db->device)) {
+			request_artwork (isource, entry, db, song);
+		}
+#else 
+		request_artwork (isource, entry, db, song);
+#endif
 		add_ipod_song_to_db (isource, db, song);
 		itdb_schedule_save (isource);
 	}
@@ -1233,12 +1328,13 @@ impl_delete_thyself (RBSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *p;
 
-	for (p = priv->playlists; p != NULL; p = p->next) {
-		RBSource *playlist = RB_SOURCE (p->data);
-		rb_source_delete_thyself (playlist);
+	for (p = priv->ipod_db->playlists; p != NULL; p = p->next) {
+		Itdb_Playlist *playlist = (Itdb_Playlist *)p->data;
+		if (!itdb_playlist_is_mpl (playlist)) {
+			RBSource *rb_playlist = RB_SOURCE (playlist->userdata);
+			rb_source_delete_thyself (rb_playlist);
+		}
 	}
-	g_list_free (priv->playlists);
-	priv->playlists = NULL;
 
 	itdb_free (priv->ipod_db);
 	priv->ipod_db = NULL;
