@@ -1,3 +1,22 @@
+# -*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-
+#
+# Copyright (C) 2006 Adam Zimmerman  <adam_zimmerman@sfu.ca>
+# Copyright (C) 2006 James Livingston  <doclivingston@gmail.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
+
 import rb, rhythmdb
 from TrackListHandler import TrackListHandler
 from BuyAlbumHandler import BuyAlbumHandler, MagnatunePurchaseError
@@ -9,6 +28,7 @@ import xml
 import urllib, zipfile
 
 has_gnome_keyring = False
+
 #try:
 #	import gnomekeyring
 #	has_gnome_keyring = True
@@ -23,7 +43,7 @@ magnatune_dir = gnome.user_dir_get() + "rhythmbox/magnatune/"
 magnatune_song_info_uri = gnomevfs.URI("http://magnatune.com/info/song_info_xml.zip")
 local_song_info_uri = gnomevfs.URI(magnatune_dir + "song_info.xml")
 local_song_info_temp_uri = gnomevfs.URI(magnatune_dir + "song_info.xml.zip.tmp")
-
+ALBUM_ART_URL = 'http://www.magnatune.com/music/%s/%s/cover.jpg'
 
 class MagnatuneSource(rb.BrowserSource):
 	__gproperties__ = {
@@ -36,11 +56,13 @@ class MagnatuneSource(rb.BrowserSource):
 	def __init__(self):
 
 		rb.BrowserSource.__init__(self, name=_("Magnatune"))
+		self.__db = None
 
 		# track data
 		self.__sku_dict = {}
 		self.__home_dict = {}
 		self.__buy_dict = {}
+		self.__art_dict = {}
 
 		# catalogue stuff
 		self.__activated = False
@@ -72,7 +94,6 @@ class MagnatuneSource(rb.BrowserSource):
 		else:
 			raise AttributeError, 'unknown property %s' % property.name
 
-
 	#
 	# RBSource methods
 	#
@@ -147,12 +168,10 @@ class MagnatuneSource(rb.BrowserSource):
 
 		rb.BrowserSource.do_impl_delete_thyself (self)
 
-
 	#
 	# methods for use by plugin and UI
 	#
 
-
 	def display_artist_info(self):
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
@@ -235,7 +254,7 @@ class MagnatuneSource(rb.BrowserSource):
 				else:
 					self.__plugin.clear_cc_details()
 
-				self.__purchase_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email)
+				self.__buy_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email)
 
 			window.destroy()
 
@@ -292,14 +311,13 @@ class MagnatuneSource(rb.BrowserSource):
 				return
 
 		parser = xml.sax.make_parser()
-		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict))
+		parser.setContentHandler(TrackListHandler(self.__db, self.__entry_type, self.__sku_dict, self.__home_dict, self.__buy_dict, self.__art_dict))
 		handle.read (64 * 1024, self.__load_catalogue_read_cb, parser)
 
 	def __load_catalogue(self):
 		self.__notify_status_changed()
 		self.__load_handle = gnomevfs.async.open (local_song_info_uri, self.__load_catalogue_open_cb)
 
-
 	def __download_update_cb (self, _reserved, info, moving):
 		self.__load_current_size = info.bytes_copied
 		self.__load_total_size = info.bytes_total
@@ -388,25 +406,11 @@ class MagnatuneSource(rb.BrowserSource):
 		if self.__notify_id == 0:
 			self.__notify_id = gobject.idle_add(change_idle_cb)
 
-
 	#
 	# internal purchasing code
 	#
-	def __purchase_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email):
+	def __buy_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email): # http://magnatune.com/info/api#purchase
 		print "purchasing tracks:", sku, pay, format, name, email
-
-		try:
-			self.__buy_album(sku, pay, format, name, email, ccnumber, ccyear, ccmonth)
-		except MagnatunePurchaseError, e:
-			error_dlg = gtk.Dialog(title="Error", flags=gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK))
-			label = gtk.Label(_("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
-			error_dlg.vbox.pack_start(label)
-			label.set_selectable(True)
-			label.show()
-			error_dlg.connect("response", lambda w, r: w.destroy())
-			error_dlg.show()
-
-	def __buy_album(self, sku, pay, format, name, email, ccnumber, ccyear, ccmonth): # http://magnatune.com/info/api#purchase
 		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
 		url = url + urllib.urlencode({
 						'id':	magnatune_partner_id,
@@ -470,9 +474,14 @@ class MagnatuneSource(rb.BrowserSource):
 				print data
 				auth_parser.feed(data)
 				handle.read(64 * 1024, self.__auth_read_cb, parser)
+			except MagnatunePurchaseError, e:
+				self.__wait_dlg.destroy()
+				rb.error_dialog(title = _("Purchase Error"),
+						message = _("An error occurred while trying to purchase the album.\nThe Magnatune server returned:\n%s") % str(e))
 			except Exception, e:
-				print e
-				raise e
+				self.__wait_dlg.destroy()
+				rb.error_dialog(title = _("Error"),
+						message = _("An error occurred while trying to purchase the album.\nThe error text is:\n%s") % str(e))
 
 	def __download_album(self, audio_dl_uri):
 			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
@@ -552,6 +561,17 @@ class MagnatuneSource(rb.BrowserSource):
 		manager = shell.get_player().get_property('ui-manager')
 		manager.get_action("/MagnatuneSourceViewPopup/MagnatuneCancelDownload").set_sensitive(False)
 
+	def playing_entry_changed (self, entry):
+		if not self.__db or not entry:
+			return
+
+		if entry.get_entry_type() != self.__db.entry_type_get_by_name("MagnatuneEntryType"):
+			return
+		
+		url = str(self.__art_dict[self.__db.entry_get(entry, rhythmdb.PROP_LOCATION)])
+		
+		self.__db.emit_entry_extra_metadata_notify (entry, 'rb:coverArt-uri', url)
+
 gobject.type_register(MagnatuneSource)
 
 
@@ -565,8 +585,6 @@ def create_if_needed(uri, mode):
 		out = gnomevfs.open(uri, open_mode=mode)
 	return out
 
-
-
 class URIIterator:
 	def __init__(self, uri):
 		self.uri_list = uri.dirname.split("/")[1:] # dirname starts with /
