@@ -38,7 +38,6 @@ G_BEGIN_DECLS
 #define RB_IS_BROWSER_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_BROWSER_SOURCE))
 #define RB_BROWSER_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_BROWSER_SOURCE, RBBrowserSourceClass))
 
-
 typedef struct RBBrowserSourcePrivate RBBrowserSourcePrivate;
 
 typedef struct
@@ -52,19 +51,20 @@ typedef struct
 {
 	RBSourceClass parent;
 
-	const char *	(*impl_get_paned_key)		(RBBrowserSource *source);
+	void		(*impl_pack_paned)		(RBBrowserSource *source, GtkWidget *paned);
+	char *		(*impl_get_paned_key)		(RBBrowserSource *source);
 	gboolean	(*impl_has_drop_support)	(RBBrowserSource *source);
+	void		(*impl_show_entry_popup)	(RBBrowserSource *source);
 } RBBrowserSourceClass;
 
-typedef gboolean	(*RBBrowserSourceFeatureFunc) (RBBrowserSource *source);
-typedef const char*	(*RBBrowserSourceStringFunc) (RBBrowserSource *source);
+typedef gboolean	(*RBBrowserSourceFeatureFunc)	(RBBrowserSource *source);
+typedef char*		(*RBBrowserSourceStringFunc)	(RBBrowserSource *source);
 
 GType		rb_browser_source_get_type		(void);
 
-const char *	rb_browser_source_get_paned_key		(RBBrowserSource *source);
+char *		rb_browser_source_get_paned_key		(RBBrowserSource *source);
 gboolean	rb_browser_source_has_drop_support	(RBBrowserSource *source);
 
-
 G_END_DECLS
 
 #endif /* __RB_BROWSER_SOURCE_H */
