@@ -57,7 +57,7 @@ static void rb_library_source_finalize (GObject *object);
 static gboolean impl_show_popup (RBSource *source);
 static GtkWidget *impl_get_config_widget (RBSource *source, RBShellPreferences *prefs);
 static char *impl_get_browser_key (RBSource *source);
-static const char *impl_get_paned_key (RBBrowserSource *source);
+static char *impl_get_paned_key (RBBrowserSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_can_paste (RBSource *asource);
 #ifdef ENABLE_TRACK_TRANSFER
@@ -222,6 +222,27 @@ rb_library_source_dispose (GObject *object)
 		source->priv->db = NULL;
 	}
 
+	if (source->priv->ui_dir_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->ui_dir_notify_id);
+		source->priv->ui_dir_notify_id = 0;
+	}
+
+	if (source->priv->library_location_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->library_location_notify_id);
+		source->priv->library_location_notify_id = 0;
+	}
+#ifdef ENABLE_TRACK_TRANSFER
+	if (source->priv->layout_path_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->layout_path_notify_id);
+		source->priv->layout_path_notify_id = 0;
+	}
+
+	if (source->priv->layout_filename_notify_id != 0) {
+		eel_gconf_notification_remove (source->priv->layout_filename_notify_id);
+		source->priv->layout_filename_notify_id = 0;
+	}
+#endif
+
 	G_OBJECT_CLASS (rb_library_source_parent_class)->dispose (object);
 }
 
@@ -238,12 +259,6 @@ rb_library_source_finalize (GObject *object)
 	g_return_if_fail (source->priv != NULL);
 
 	rb_debug ("finalizing library source");
-	eel_gconf_notification_remove (source->priv->ui_dir_notify_id);
-	eel_gconf_notification_remove (source->priv->library_location_notify_id);
-#ifdef ENABLE_TRACK_TRANSFER
-	eel_gconf_notification_remove (source->priv->layout_path_notify_id);
-	eel_gconf_notification_remove (source->priv->layout_filename_notify_id);
-#endif
 
 	G_OBJECT_CLASS (rb_library_source_parent_class)->finalize (object);
 }
@@ -266,6 +281,7 @@ rb_library_source_constructor (GType type,
 	RBLibrarySource *source;
 	RBShell *shell;
 	RBEntryView *songs;
+	GSList *list;
 
 	source = RB_LIBRARY_SOURCE (G_OBJECT_CLASS (rb_library_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
@@ -275,6 +291,15 @@ rb_library_source_constructor (GType type,
 
 	rb_library_source_ui_prefs_sync (source);
 
+	/* Set up the default library location if there's no library location set */
+	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
+	if (g_slist_length (list) == 0) {
+		list = g_slist_prepend (list, g_strdup (rb_music_dir ()));
+		eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+	}
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+
 	source->priv->library_location_notify_id =
 		eel_gconf_notification_add (CONF_LIBRARY_LOCATION,
 				    (GConfClientNotifyFunc) rb_library_source_library_location_changed, source);
@@ -306,14 +331,15 @@ rb_library_source_new (RBShell *shell)
 
 	entry_type = RHYTHMDB_ENTRY_TYPE_SONG;
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-					 "stock_music-library",
+					 "audio-x-generic",
 					 size,
 					 0, NULL);
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
-					  "name", _("Library"),
+					  "name", _("Music"),
 					  "entry-type", entry_type,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  "sorting-key", CONF_STATE_LIBRARY_SORTING,
 					  "shell", shell,
 					  "icon", icon,
@@ -593,10 +619,6 @@ rb_library_source_library_location_cb (GtkEntry *entry,
 	if (list)
 		g_slist_free (list);
 
-	/* don't do the first-run druid if the user sets the library location */
-	if (list)
-		eel_gconf_set_boolean (CONF_FIRST_TIME, TRUE);
-
 	return FALSE;
 }
 
@@ -615,10 +637,10 @@ impl_get_browser_key (RBSource *source)
 	return g_strdup (CONF_STATE_SHOW_BROWSER);
 }
 
-static const char *
+static char *
 impl_get_paned_key (RBBrowserSource *status)
 {
-	return CONF_STATE_PANED_POSITION;
+	return g_strdup (CONF_STATE_PANED_POSITION);
 }
 
 static void
@@ -1246,7 +1268,8 @@ impl_want_uri (RBSource *source, const char *uri)
 	/* assume anything local, on smb, or on sftp is a song */
 	if (rb_uri_is_local (uri) ||
 	    g_str_has_prefix (uri, "smb://") ||
-	    g_str_has_prefix (uri, "sftp://"))
+	    g_str_has_prefix (uri, "sftp://") ||
+	    g_str_has_prefix (uri, "ssh://"))
 		return 50;
 
 	return 0;
