@@ -271,7 +271,7 @@ rb_playlist_source_constructor (GType type,
 							 rb_playlist_source_track_cell_data_func,
 							 source, NULL);
 		rb_entry_view_insert_column_custom (source->priv->songs, column, title,
-						    "PlaylistTrack", NULL, 0, 0);
+						    "PlaylistTrack", NULL, 0, NULL, 0);
 	}
 
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER, FALSE);
