@@ -206,7 +206,10 @@ sexy_icon_entry_map(GtkWidget *widget)
 		GTK_WIDGET_CLASS(parent_class)->map(widget);
 
 		for (i = 0; i < MAX_ICONS; i++)
-			gdk_window_show(entry->priv->icons[i].window);
+		{
+			if (entry->priv->icons[i].icon != NULL)
+				gdk_window_show(entry->priv->icons[i].window);
+		}
 	}
 }
 
@@ -219,7 +222,10 @@ sexy_icon_entry_unmap(GtkWidget *widget)
 		int i;
 
 		for (i = 0; i < MAX_ICONS; i++)
-			gdk_window_hide(entry->priv->icons[i].window);
+		{
+			if (entry->priv->icons[i].icon != NULL)
+				gdk_window_hide(entry->priv->icons[i].window);
+		}
 
 		GTK_WIDGET_CLASS(parent_class)->unmap(widget);
 	}
@@ -331,23 +337,24 @@ sexy_icon_entry_realize(GtkWidget *widget)
 
 	GTK_WIDGET_CLASS(parent_class)->realize(widget);
 
+	attributes.width = 1;
+	attributes.height = 1;
+	attributes.window_type = GDK_WINDOW_CHILD;
+	attributes.wclass = GDK_INPUT_OUTPUT;
+	attributes.visual = gtk_widget_get_visual(widget);
+	attributes.colormap = gtk_widget_get_colormap(widget);
+	attributes.event_mask = gtk_widget_get_events(widget);
+	attributes.event_mask |=
+		(GDK_EXPOSURE_MASK
+		 | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
+		 | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+
+	attributes_mask = GDK_WA_VISUAL | GDK_WA_COLORMAP;
+
 	for (i = 0; i < MAX_ICONS; i++)
 	{
 		SexyIconInfo *icon_info;
 
-		attributes.window_type = GDK_WINDOW_CHILD;
-		attributes.wclass = GDK_INPUT_OUTPUT;
-		attributes.visual = gtk_widget_get_visual(widget);
-		attributes.colormap = gtk_widget_get_colormap(widget);
-		attributes.event_mask = gtk_widget_get_events(widget);
-		attributes.event_mask |=
-			(GDK_EXPOSURE_MASK
-			 | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
-			 | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
-
-		attributes_mask = GDK_WA_X | GDK_WA_Y |
-		                  GDK_WA_VISUAL | GDK_WA_COLORMAP;
-
 		icon_info = &entry->priv->icons[i];
 		icon_info->window = gdk_window_new(widget->window, &attributes,
 										   attributes_mask);
@@ -538,13 +545,15 @@ draw_icon(GtkWidget *widget, SexyIconEntryPosition icon_pos)
 	GdkPixbuf *pixbuf;
 	gint x, y, width, height;
 
-	if (icon_info->icon == NULL)
+	if (icon_info->icon == NULL || !GTK_WIDGET_REALIZED(widget))
 		return;
 
 	if ((pixbuf = get_pixbuf_from_icon(entry, icon_pos)) == NULL)
 		return;
 
 	gdk_drawable_get_size(icon_info->window, &width, &height);
+	if ((width < 2 * ICON_MARGIN) || (height < 2 * ICON_MARGIN))
+		return;
 
 	if (gdk_pixbuf_get_height(pixbuf) > height)
 	{
@@ -814,15 +823,24 @@ sexy_icon_entry_set_icon(SexyIconEntry *entry, SexyIconEntryPosition icon_pos,
 		{
 			gtk_widget_destroy(GTK_WIDGET(icon_info->icon));
 			icon_info->icon = NULL;
+
+			/*
+			 * Explicitly check, as the pointer may become invalidated
+			 * during destruction.
+			 */
+			if (icon_info->window != NULL && GDK_IS_WINDOW(icon_info->window))
+				gdk_window_hide(icon_info->window);
 		}
 	}
 	else
 	{
+		if (icon_info->window != NULL && icon_info->icon == NULL)
+			gdk_window_show(icon_info->window);
+
 		g_signal_connect(G_OBJECT(icon), "notify",
 						 G_CALLBACK(update_icon), entry);
 
 		icon_info->icon = icon;
-
 		g_object_ref(icon);
 	}
 
