@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #ifndef __RB_CELL_RENDERER_PIXBUF_H__
@@ -51,7 +51,7 @@ struct _RBCellRendererPixbufClass
 {
   GtkCellRendererClass parent_class;
 
-  void (*pixbuf_clicked) (GtkTreePath *path);
+  void (*pixbuf_clicked) (RBCellRendererPixbuf *renderer, GtkTreePath *path);
 };
 
 GtkType          rb_cell_renderer_pixbuf_get_type (void);
