@@ -41,6 +41,7 @@
 
 static void rb_property_view_class_init (RBPropertyViewClass *klass);
 static void rb_property_view_init (RBPropertyView *view);
+static void rb_property_view_dispose (GObject *object);
 static void rb_property_view_finalize (GObject *object);
 static void rb_property_view_set_property (GObject *object,
 					   guint prop_id,
@@ -110,26 +111,6 @@ enum
 
 static guint rb_property_view_signals[LAST_SIGNAL] = { 0 };
 
-enum {
-	TARGET_ALBUMS,
-	TARGET_GENRE,
-	TARGET_ARTISTS,
-	TARGET_URIS,
-};
-
-static const GtkTargetEntry targets_album  [] = {
-	{ "text/x-rhythmbox-album",  0, TARGET_ALBUMS },
-	{ "text/uri-list", 0, TARGET_URIS },
-};
-static const GtkTargetEntry targets_genre  [] = {
-	{ "text/x-rhythmbox-genre",  0, TARGET_GENRE },
-	{ "text/uri-list", 0, TARGET_URIS },
-};
-static const GtkTargetEntry targets_artist [] = {
-	{ "text/x-rhythmbox-artist", 0, TARGET_ARTISTS },
-	{ "text/uri-list", 0, TARGET_URIS },
-};
-
 G_DEFINE_TYPE (RBPropertyView, rb_property_view, GTK_TYPE_SCROLLED_WINDOW)
 
 static void
@@ -137,6 +118,7 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = rb_property_view_dispose;
 	object_class->finalize = rb_property_view_finalize;
 	object_class->constructor = rb_property_view_constructor;
 
@@ -294,7 +276,7 @@ rb_property_view_set_model_internal (RBPropertyView *view,
 }
 
 static void
-rb_property_view_finalize (GObject *object)
+rb_property_view_dispose (GObject *object)
 {
 	RBPropertyView *view;
 
@@ -303,10 +285,21 @@ rb_property_view_finalize (GObject *object)
 
 	view = RB_PROPERTY_VIEW (object);
 
-	g_return_if_fail (view->priv != NULL);
-
 	rb_property_view_set_model_internal (view, NULL);
 
+	G_OBJECT_CLASS (rb_property_view_parent_class)->dispose (object);
+}
+
+static void
+rb_property_view_finalize (GObject *object)
+{
+	RBPropertyView *view;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_PROPERTY_VIEW (object));
+
+	view = RB_PROPERTY_VIEW (object);
+
 	g_free (view->priv->title);
 
 	G_OBJECT_CLASS (rb_property_view_parent_class)->finalize (object);
@@ -508,16 +501,16 @@ rb_property_view_cell_data_func (GtkTreeViewColumn *column,
 
 		switch (view->priv->propid) {
 		case RHYTHMDB_PROP_ARTIST:
-			fmt = ngettext ("All %d artist (%d)", "All %d artists (%d)", nodes);
+			fmt = ngettext ("%d artist (%d)", "All %d artists (%d)", nodes);
 			break;
 		case RHYTHMDB_PROP_ALBUM:
-			fmt = ngettext ("All %d album (%d)", "All %d albums (%d)", nodes);
+			fmt = ngettext ("%d album (%d)", "All %d albums (%d)", nodes);
 			break;
 		case RHYTHMDB_PROP_GENRE:
-			fmt = ngettext ("All %d genre (%d)", "All %d genres (%d)", nodes);
+			fmt = ngettext ("%d genre (%d)", "All %d genres (%d)", nodes);
 			break;
 		default:
-			fmt = _("All %d (%d)");
+			fmt = ngettext ("%d (%d)", "All %d (%d)", nodes);
 			break;
 		}
 
