@@ -25,10 +25,13 @@
  * Yes, this code is ugly.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <time.h>
+#include <math.h>
+
+#define EPSILON 0.0001
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -53,14 +56,15 @@ static GObject *rb_song_info_constructor (GType type, guint n_construct_properti
 					  GObjectConstructParam *construct_properties);
 
 static void rb_song_info_show (GtkWidget *widget);
+static void rb_song_info_dispose (GObject *object);
 static void rb_song_info_finalize (GObject *object);
-static void rb_song_info_set_property (GObject *object, 
+static void rb_song_info_set_property (GObject *object,
 				       guint prop_id,
-				       const GValue *value, 
+				       const GValue *value,
 				       GParamSpec *pspec);
-static void rb_song_info_get_property (GObject *object, 
+static void rb_song_info_get_property (GObject *object,
 				       guint prop_id,
-				       GValue *value, 
+				       GValue *value,
 				       GParamSpec *pspec);
 static void rb_song_info_response_cb (GtkDialog *dialog,
 				      int response_id,
@@ -142,7 +146,7 @@ enum
 	LAST_SIGNAL
 };
 
-enum 
+enum
 {
 	PROP_0,
 	PROP_SOURCE,
@@ -182,11 +186,13 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_CURRENT_ENTRY,
-					 g_param_spec_pointer ("current-entry",
-					                       "RhythmDBEntry",
-					                       "RhythmDBEntry object",
-					                       G_PARAM_READABLE));
+					 g_param_spec_boxed ("current-entry",
+					                     "RhythmDBEntry",
+					                     "RhythmDBEntry object",
+							     RHYTHMDB_TYPE_ENTRY,
+					                     G_PARAM_READABLE));
 
+	object_class->dispose = rb_song_info_dispose;
 	object_class->finalize = rb_song_info_finalize;
 
 	rb_song_info_signals[PRE_METADATA_CHANGE] =
@@ -195,10 +201,10 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSongInfoClass, pre_metadata_change),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
+			      g_cclosure_marshal_VOID__BOXED,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_POINTER);
+			      RHYTHMDB_TYPE_ENTRY);
 
 	rb_song_info_signals[POST_METADATA_CHANGE] =
 		g_signal_new ("post-metadata-change",
@@ -206,10 +212,10 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBSongInfoClass, post_metadata_change),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
+			      g_cclosure_marshal_VOID__BOXED,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_POINTER);
+			      RHYTHMDB_TYPE_ENTRY);
 
 	g_type_class_add_private (klass, sizeof (RBSongInfoPrivate));
 }
@@ -228,6 +234,7 @@ rb_song_info_init (RBSongInfo *song_info)
 	gtk_dialog_set_has_separator (GTK_DIALOG (song_info), FALSE);
 
 	gtk_container_set_border_width (GTK_CONTAINER (song_info), 5);
+	gtk_window_set_resizable (GTK_WINDOW (song_info), TRUE);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (song_info)->vbox), 2);
 }
 
@@ -247,16 +254,16 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	song_info->priv->backward = gtk_dialog_add_button (GTK_DIALOG (song_info),
 							   GTK_STOCK_GO_BACK,
 							   GTK_RESPONSE_NONE);
-	
+
 	g_signal_connect_object (G_OBJECT (song_info->priv->backward),
 				 "clicked",
 				 G_CALLBACK (rb_song_info_backward_clicked_cb),
 				 song_info, 0);
-	
+
 	song_info->priv->forward = gtk_dialog_add_button (GTK_DIALOG (song_info),
 							   GTK_STOCK_GO_FORWARD,
 							   GTK_RESPONSE_NONE);
-	
+
 	g_signal_connect_object (G_OBJECT (song_info->priv->forward),
 				 "clicked",
 				 G_CALLBACK (rb_song_info_forward_clicked_cb),
@@ -348,17 +355,21 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 
 	g_return_val_if_fail (selected_entries != NULL, NULL);
 
-	for (tem = selected_entries; tem; tem = tem->next)
+	for (tem = selected_entries; tem; tem = tem->next) {
 		if (!rhythmdb_entry_is_editable (song_info->priv->db,
 						 selected_entries->data)) {
 			editable = FALSE;
 			break;
 		}
+	}
+
 	song_info->priv->editable = editable;
 
 	if (selected_entries->next == NULL) {
 		song_info->priv->current_entry = selected_entries->data;
 		song_info->priv->selected_entries = NULL;
+
+		g_list_foreach (selected_entries, (GFunc)rhythmdb_entry_unref, NULL);
 		g_list_free (selected_entries);
 	} else {
 		song_info->priv->current_entry = NULL;
@@ -378,7 +389,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
 				   glade_xml_get_widget (xml, "song_info_notebook"));
 	}
-	
+
 	glade_xml_signal_autoconnect (xml);
 
 	song_info->priv->artist = glade_xml_get_widget (xml, "song_info_artist");
@@ -392,7 +403,7 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->genre), song_info->priv->genres);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->artist), song_info->priv->artists);
 	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->album), song_info->priv->albums);
-	
+
 	rb_glade_boldify_label (xml, "album_label");
 	rb_glade_boldify_label (xml, "artist_label");
 	rb_glade_boldify_label (xml, "genre_label");
@@ -458,6 +469,50 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	return G_OBJECT (song_info);
 }
 
+static void
+rb_song_info_dispose (GObject *object)
+{
+	RBSongInfo *song_info;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_SONG_INFO (object));
+
+	song_info = RB_SONG_INFO (object);
+
+	g_return_if_fail (song_info->priv != NULL);
+
+	if (song_info->priv->albums != NULL) {
+		g_object_unref (song_info->priv->albums);
+		song_info->priv->albums = NULL;
+	}
+	if (song_info->priv->artists != NULL) {
+		g_object_unref (song_info->priv->artists);
+		song_info->priv->artists = NULL;
+	}
+	if (song_info->priv->genres != NULL) {
+		g_object_unref (song_info->priv->genres);
+		song_info->priv->genres = NULL;
+	}
+
+	if (song_info->priv->db != NULL) {
+		g_object_unref (song_info->priv->db);
+		song_info->priv->db = NULL;
+	}
+	if (song_info->priv->source != NULL) {
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      G_CALLBACK (rb_song_info_query_model_changed_cb),
+						      song_info);
+		g_object_unref (song_info->priv->source);
+		song_info->priv->source = NULL;
+	}
+	if (song_info->priv->query_model != NULL) {
+		g_object_unref (song_info->priv->query_model);
+		song_info->priv->query_model = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_song_info_parent_class)->dispose (object);
+}
+
 static void
 rb_song_info_finalize (GObject *object)
 {
@@ -470,21 +525,78 @@ rb_song_info_finalize (GObject *object)
 
 	g_return_if_fail (song_info->priv != NULL);
 
-	g_signal_handlers_disconnect_by_func (G_OBJECT (song_info->priv->source),
-					      G_CALLBACK (rb_song_info_query_model_changed_cb),
-					      song_info);
-
-	g_object_unref (song_info->priv->albums);
-	g_object_unref (song_info->priv->artists);
-	g_object_unref (song_info->priv->genres);
-
-	g_object_unref (G_OBJECT (song_info->priv->db));
-	g_object_unref (G_OBJECT (song_info->priv->source));
-	g_object_unref (G_OBJECT (song_info->priv->query_model));
+	if (song_info->priv->selected_entries != NULL) {
+		g_list_foreach (song_info->priv->selected_entries, (GFunc)rhythmdb_entry_unref, NULL);
+		g_list_free (song_info->priv->selected_entries);
+	}
 
 	G_OBJECT_CLASS (rb_song_info_parent_class)->finalize (object);
 }
 
+static void
+rb_song_info_set_source_internal (RBSongInfo *song_info,
+				  RBSource   *source)
+{
+	RhythmDB *old_db = song_info->priv->db;
+
+	if (song_info->priv->source != NULL) {
+		g_signal_handlers_disconnect_by_func (song_info->priv->source,
+						      rb_song_info_query_model_changed_cb,
+						      song_info);
+		g_object_unref (song_info->priv->source);
+		g_object_unref (song_info->priv->query_model);
+		g_object_unref (song_info->priv->db);
+	}
+
+	song_info->priv->source = source;
+
+	g_object_ref (song_info->priv->source);
+
+	g_object_get (G_OBJECT (song_info->priv->source), "query-model", &song_info->priv->query_model, NULL);
+
+	g_signal_connect_object (G_OBJECT (song_info->priv->source),
+				 "notify::query-model",
+				 G_CALLBACK (rb_song_info_query_model_changed_cb),
+				 song_info, 0);
+
+	g_object_get (G_OBJECT (song_info->priv->query_model), "db", &song_info->priv->db, NULL);
+
+	if (old_db != song_info->priv->db) {
+		if (song_info->priv->albums) {
+			g_object_unref (song_info->priv->albums);
+		}
+		if (song_info->priv->artists) {
+			g_object_unref (song_info->priv->artists);
+		}
+		if (song_info->priv->genres) {
+			g_object_unref (song_info->priv->genres);
+		}
+
+		song_info->priv->albums  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ALBUM);
+		song_info->priv->artists = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_ARTIST);
+		song_info->priv->genres  = rhythmdb_property_model_new (song_info->priv->db, RHYTHMDB_PROP_GENRE);
+
+		g_object_set (song_info->priv->albums,  "query-model", song_info->priv->query_model, NULL);
+		g_object_set (song_info->priv->artists, "query-model", song_info->priv->query_model, NULL);
+		g_object_set (song_info->priv->genres,  "query-model", song_info->priv->query_model, NULL);
+
+		if(song_info->priv->album) {
+			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
+			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
+		}
+
+		if(song_info->priv->artist) {
+			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
+			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
+		}
+
+		if(song_info->priv->genre) {
+			GtkEntryCompletion *comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
+			gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
+		}
+	}
+}
+
 static void
 rb_song_info_set_property (GObject *object,
 			   guint prop_id,
@@ -493,60 +605,10 @@ rb_song_info_set_property (GObject *object,
 {
 	RBSongInfo *song_info = RB_SONG_INFO (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_SOURCE:
-	{
-		RhythmDB* old_db = song_info->priv->db;
-
-		song_info->priv->source = g_value_get_object (value);
-		g_object_ref (G_OBJECT (song_info->priv->source));
-		g_object_get (G_OBJECT (song_info->priv->source),
-			      "query-model", &song_info->priv->query_model, NULL);
-		g_signal_connect_object (G_OBJECT (song_info->priv->source),
-					 "notify::query-model",
-					 G_CALLBACK (rb_song_info_query_model_changed_cb),
-					 song_info, 0);
-
-		g_object_get (G_OBJECT (song_info->priv->query_model), "db",
-			      &song_info->priv->db, NULL);
-
-		if(old_db != song_info->priv->db) {
-			if(song_info->priv->albums) {
-				g_object_unref(song_info->priv->albums);
-			}
-			if(song_info->priv->artists) {
-				g_object_unref(song_info->priv->artists);
-			}
-			if(song_info->priv->genres) {
-				g_object_unref(song_info->priv->genres);
-			}
-
-			song_info->priv->albums  = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_ALBUM);
-			song_info->priv->artists = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_ARTIST);
-			song_info->priv->genres  = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_GENRE);
-
-			g_object_set(song_info->priv->albums,  "query-model", song_info->priv->query_model, NULL);
-			g_object_set(song_info->priv->artists, "query-model", song_info->priv->query_model, NULL);
-			g_object_set(song_info->priv->genres,  "query-model", song_info->priv->query_model, NULL);
-
-			if(song_info->priv->album) {
-				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
-				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
-			}
-
-			if(song_info->priv->artist) {
-				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
-				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
-			}
-
-			if(song_info->priv->genre) {
-				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
-				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
-			}
-		}
-	}
-	break;
+		rb_song_info_set_source_internal (song_info, g_value_get_object (value));
+		break;
 	case PROP_ENTRY_VIEW:
 		song_info->priv->entry_view = g_value_get_object (value);
 		break;
@@ -564,8 +626,7 @@ rb_song_info_get_property (GObject *object,
 {
 	RBSongInfo *song_info = RB_SONG_INFO (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_SOURCE:
 		g_value_set_object (value, song_info->priv->source);
 		break;
@@ -573,7 +634,7 @@ rb_song_info_get_property (GObject *object,
 		g_value_set_object (value, song_info->priv->entry_view);
 		break;
 	case PROP_CURRENT_ENTRY:
-		g_value_set_pointer (value, song_info->priv->current_entry);
+		g_value_set_boxed (value, song_info->priv->current_entry);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -590,13 +651,13 @@ rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 	if (!entry_view)
 		entry_view = rb_source_get_entry_view (source);
 
-	if (rb_entry_view_have_selection (entry_view) == FALSE) 
+	if (rb_entry_view_have_selection (entry_view) == FALSE)
 		return NULL;
 
 	/* create the dialog */
-	song_info = g_object_new (RB_TYPE_SONG_INFO, 
-				  "source", source, 
-				  "entry-view", entry_view, 
+	song_info = g_object_new (RB_TYPE_SONG_INFO,
+				  "source", source,
+				  "entry-view", entry_view,
 				  NULL);
 
 	g_return_val_if_fail (song_info->priv != NULL, NULL);
@@ -661,7 +722,6 @@ rb_song_info_set_entry_rating (RBSongInfo *info,
 	rhythmdb_entry_set (info->priv->db, entry, RHYTHMDB_PROP_RATING, &value);
 	g_value_unset (&value);
 }
-	
 
 static void
 rb_song_info_rated_cb (RBRating *rating,
@@ -708,7 +768,7 @@ rb_song_info_populate_num_field (GtkEntry *field, gulong num)
 	g_free (tmp);
 }
 
-static void 
+static void
 rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 {
 	gboolean mixed_artists = FALSE;
@@ -716,11 +776,13 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 	gboolean mixed_genres = FALSE;
 	gboolean mixed_years = FALSE;
 	gboolean mixed_disc_numbers = FALSE;
+	gboolean mixed_ratings = FALSE;
 	const char *artist = NULL;
 	const char *album = NULL;
 	const char *genre = NULL;
 	int year = 0;
 	int disc_number = 0;
+	double rating = 0.0; /* Zero is used for both "unrated" and "mixed ratings" too */
 	GList *l;
 
 	g_assert (song_info->priv->selected_entries);
@@ -732,6 +794,7 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		const char *entry_genre;
 		int entry_year;
 		int entry_disc_number;
+		double entry_rating;
 
 		entry = (RhythmDBEntry*)l->data;
 		entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
@@ -739,6 +802,7 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
 		entry_year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
 		entry_disc_number = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
+		entry_rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
 
 		/* grab first valid values */
 		if (artist == NULL)
@@ -751,6 +815,8 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 			year = entry_year;
 		if (disc_number == 0)
 			disc_number = entry_disc_number;
+		if (fabs(rating) < EPSILON)
+			rating = entry_rating;
 
 		/* locate mixed values */
 		if (artist != entry_artist)
@@ -763,10 +829,12 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 			mixed_years = TRUE;
 		if (disc_number != entry_disc_number)
 			mixed_disc_numbers = TRUE;
+		if (fabs(rating - entry_rating) >= EPSILON)
+			mixed_ratings = TRUE;
 
 		/* don't continue search if everything is mixed */
 		if (mixed_artists && mixed_albums && mixed_genres &&
-		    mixed_years && mixed_disc_numbers)
+		    mixed_years && mixed_disc_numbers && mixed_ratings)
 			break;
 	}
 
@@ -780,15 +848,17 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->year), year);
 	if (!mixed_disc_numbers && disc_number != 0)
 		rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur), disc_number);
+	if (!mixed_ratings && fabs(rating) >= EPSILON)
+		g_object_set (G_OBJECT (song_info->priv->rating), "rating", rating, NULL);
 }
 
-static void 
+static void
 rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
 	const char *text;
 	char *tmp;
 	gulong num;
-	
+
 	g_assert (song_info->priv->current_entry);
 
 	/* update the buttons sensitivity */
@@ -796,7 +866,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 
 	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_TITLE);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
-	
+
 	tmp = g_strdup_printf (_("%s Properties"), text);
 	gtk_window_set_title (GTK_WINDOW (song_info), tmp);
 	g_free (tmp);
@@ -900,22 +970,22 @@ rb_song_info_update_location (RBSongInfo *song_info)
 	if (text != NULL) {
 		char *tmp, *tmp_utf8;
 		char *basename, *dir, *desktopdir;
-		
+
 		basename = g_path_get_basename (text);
 		tmp = gnome_vfs_unescape_string_for_display (basename);
 		g_free (basename);
 		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
 		g_free (tmp);
 		if (tmp_utf8 != NULL) {
-			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), 
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name),
 					    tmp_utf8);
 		} else {
-			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), 
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name),
 					    _("Unknown file name"));
 		}
 
 		g_free (tmp_utf8);
-	
+
 		tmp = gnome_vfs_get_local_path_from_uri (text);
 		if (tmp == NULL)
 			tmp = g_strdup (text);
@@ -935,10 +1005,10 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		g_free (desktopdir);
 
 		if (tmp_utf8 != NULL) {
-			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), 
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location),
 					    tmp_utf8);
 		} else {
-			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), 
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location),
 					    _("Unknown location"));
 		}
 		g_free (tmp_utf8);
@@ -952,9 +1022,8 @@ rb_song_info_backward_clicked_cb (GtkWidget *button,
 	RhythmDBEntry *new_entry;
 
 	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
-	new_entry = 
-		rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
-							      song_info->priv->current_entry);
+	new_entry = rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
+								  song_info->priv->current_entry);
 	g_return_if_fail (new_entry != NULL);
 
 	song_info->priv->current_entry = new_entry;
@@ -963,6 +1032,7 @@ rb_song_info_backward_clicked_cb (GtkWidget *button,
 
 	rb_song_info_populate_dialog (song_info);
 	g_object_notify (G_OBJECT (song_info), "current-entry");
+	rhythmdb_entry_unref (new_entry);
 }
 
 static void
@@ -972,9 +1042,8 @@ rb_song_info_forward_clicked_cb (GtkWidget *button,
 	RhythmDBEntry *new_entry;
 
 	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
-	new_entry = 
-		rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
-							  song_info->priv->current_entry);
+	new_entry = rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
+							      song_info->priv->current_entry);
 	g_return_if_fail (new_entry != NULL);
 
 	song_info->priv->current_entry = new_entry;
@@ -983,6 +1052,8 @@ rb_song_info_forward_clicked_cb (GtkWidget *button,
 
 	rb_song_info_populate_dialog (song_info);
 	g_object_notify (G_OBJECT (song_info), "current-entry");
+
+	rhythmdb_entry_unref (new_entry);
 }
 
 /*
@@ -998,17 +1069,22 @@ rb_song_info_update_buttons (RBSongInfo *song_info)
 
 	if (!song_info->priv->current_entry)
 		return;
-	
+
 	/* backward */
 	entry = rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
 							      song_info->priv->current_entry);
-	
+
 	gtk_widget_set_sensitive (song_info->priv->backward, entry != NULL);
+	if (entry != NULL)
+		rhythmdb_entry_unref (entry);
+
 	/* forward */
 	entry = rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
 							  song_info->priv->current_entry);
-	
+
 	gtk_widget_set_sensitive (song_info->priv->forward, entry != NULL);
+	if (entry != NULL)
+		rhythmdb_entry_unref (entry);
 }
 
 static void
@@ -1071,7 +1147,7 @@ rb_song_info_query_model_changed_cb (GObject *source,
 	g_signal_connect_object (G_OBJECT (song_info->priv->query_model),
 				 "rows-reordered", G_CALLBACK (rb_song_info_query_model_reordered_cb),
 				 song_info, 0);
-	
+
 	/* update next button sensitivity */
 	rb_song_info_update_buttons (song_info);
 }
@@ -1106,7 +1182,7 @@ rb_song_info_update_rating (RBSongInfo *song_info)
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
 	rating = rhythmdb_entry_get_double (song_info->priv->current_entry, RHYTHMDB_PROP_RATING);
-	g_object_set (G_OBJECT (song_info->priv->rating),
+	g_object_set (song_info->priv->rating,
 		      "rating", rating,
 		      NULL);
 }
@@ -1132,7 +1208,7 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 {
 	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
 	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
-	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));
 	const char *year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
 	const char *discn_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
 
@@ -1149,10 +1225,10 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_set_string (&val, album);
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
 			const char *entry_album;
-			
+
 			entry = (RhythmDBEntry *)tem->data;
 			entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
-			
+
 			if (strcmp (album, entry_album) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
@@ -1161,16 +1237,16 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		}
 		g_value_unset (&val);
 	}
-	
+
 	if (strlen (artist) > 0) {
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, artist);
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
 			const char *entry_artist;
-			
+
 			entry = (RhythmDBEntry *)tem->data;
 			entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
-			
+
 			if (strcmp (artist, entry_artist) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
@@ -1188,7 +1264,7 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 
 			entry = (RhythmDBEntry *)tem->data;
 			entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
-			
+
 			if (strcmp (genre, entry_genre) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
@@ -1224,7 +1300,7 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 			g_date_free (date);
 
 	}
-	
+
 	discn = g_ascii_strtoull (discn_str, &endptr, 10);
 	if (endptr != discn_str) {
 		GType type;
@@ -1235,7 +1311,7 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
 			gulong entry_disc_num;
-			
+
 			entry = (RhythmDBEntry *)tem->data;
 			entry_disc_num = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 
@@ -1248,7 +1324,6 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_unset (&val);
 	}
 
-
 	if (changed)
 		rhythmdb_commit (dialog->priv->db);
 }
@@ -1259,7 +1334,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	const char *title;
 	const char *genre;
 	const char *artist;
-	const char *album;	
+	const char *album;
 	const char *tracknum_str;
 	const char *discnum_str;
 	const char *year_str;
@@ -1277,12 +1352,12 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	title = gtk_entry_get_text (GTK_ENTRY (dialog->priv->title));
 	genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
 	artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
-	album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));
 	tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
 	discnum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
 	year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
 
-	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[PRE_METADATA_CHANGE], 0,
+	g_signal_emit (dialog, rb_song_info_signals[PRE_METADATA_CHANGE], 0,
 		       entry);
 
 	tracknum = g_ascii_strtoull (tracknum_str, &endptr, 10);
@@ -1296,7 +1371,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		g_value_unset (&val);
 		changed = TRUE;
 	}
-	
+
 	discnum = g_ascii_strtoull (discnum_str, &endptr, 10);
 	entry_val = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 	if ((endptr != discnum_str) && (discnum != entry_val)) {
@@ -1308,14 +1383,14 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		g_value_unset (&val);
 		changed = TRUE;
 	}
-	
+
 	year = g_ascii_strtoull (year_str, &endptr, 10);
 	entry_val = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
-	if ((endptr != year_str) && 
-	    (year != entry_val || 
+	if ((endptr != year_str) &&
+	    (year != entry_val ||
 	     (entry_val == 0 && year > 0))) {
 		GDate *date = NULL;
-	
+
 		if (year > 0) {
 			if (entry_val > 0) {
 				gulong julian;
@@ -1334,13 +1409,12 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		g_value_set_ulong (&val, (date ? g_date_get_julian (date) : 0));
 		rhythmdb_entry_set (dialog->priv->db, entry, RHYTHMDB_PROP_DATE, &val);
 		changed = TRUE;
-		
+
 		g_value_unset (&val);
 		if (date)
 			g_date_free (date);
 	}
 
-
 	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
 	if (strcmp (title, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
@@ -1397,7 +1471,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 
 	if (changed)
 		rhythmdb_commit (dialog->priv->db);
-}	
+}
 
 static void
 rb_song_info_sync_entries (RBSongInfo *dialog)
